# -*- coding: utf-8 -*-

################################################################################
## Form generated from reading UI file 'details.ui'
##
## Created by: Qt User Interface Compiler version 6.7.2
##
## WARNING! All changes made in this file will be lost when recompiling UI file!
################################################################################

from PySide6.QtCore import (QCoreApplication, QDate, QDateTime, QLocale,
    QMetaObject, QObject, QPoint, QRect,
    QSize, QTime, QUrl, Qt)
from PySide6.QtGui import (QBrush, QColor, QConicalGradient, QCursor,
    QFont, QFontDatabase, QGradient, QIcon,
    QImage, QKeySequence, QLinearGradient, QPainter,
    QPalette, QPixmap, QRadialGradient, QTransform)
from PySide6.QtWidgets import (QApplication, QCheckBox, QFormLayout, QFrame,
    QGridLayout, QGroupBox, QHBoxLayout, QLabel,
    QLayout, QLineEdit, QProgressBar, QPushButton,
    QSizePolicy, QSpacerItem, QStackedWidget, QVBoxLayout,
    QWidget)

class Ui_GameDetails(object):
    def setupUi(self, GameDetails):
        if not GameDetails.objectName():
            GameDetails.setObjectName(u"GameDetails")
        GameDetails.resize(814, 470)
        GameDetails.setWindowTitle(u"GameDetails")
        self.main_layout = QHBoxLayout(GameDetails)
        self.main_layout.setObjectName(u"main_layout")
        self.left_layout = QVBoxLayout()
        self.left_layout.setObjectName(u"left_layout")
        self.tags_group = QGroupBox(GameDetails)
        self.tags_group.setObjectName(u"tags_group")
        sizePolicy = QSizePolicy(QSizePolicy.Policy.Preferred, QSizePolicy.Policy.Fixed)
        sizePolicy.setHorizontalStretch(0)
        sizePolicy.setVerticalStretch(0)
        sizePolicy.setHeightForWidth(self.tags_group.sizePolicy().hasHeightForWidth())
        self.tags_group.setSizePolicy(sizePolicy)
        self.tags_layout = QGridLayout(self.tags_group)
        self.tags_layout.setObjectName(u"tags_layout")
        self.tags_layout.setHorizontalSpacing(0)
        self.completed_check = QCheckBox(self.tags_group)
        self.completed_check.setObjectName(u"completed_check")

        self.tags_layout.addWidget(self.completed_check, 3, 0, 1, 2)

        self.hidden_check = QCheckBox(self.tags_group)
        self.hidden_check.setObjectName(u"hidden_check")

        self.tags_layout.addWidget(self.hidden_check, 0, 0, 1, 2)

        self.custom1_edit = QLineEdit(self.tags_group)
        self.custom1_edit.setObjectName(u"custom1_edit")

        self.tags_layout.addWidget(self.custom1_edit, 4, 1, 1, 1)

        self.favorites_check = QCheckBox(self.tags_group)
        self.favorites_check.setObjectName(u"favorites_check")

        self.tags_layout.addWidget(self.favorites_check, 1, 0, 1, 2)

        self.custom1_check = QCheckBox(self.tags_group)
        self.custom1_check.setObjectName(u"custom1_check")
        self.custom1_check.setText(u"")

        self.tags_layout.addWidget(self.custom1_check, 4, 0, 1, 1)

        self.backlog_check = QCheckBox(self.tags_group)
        self.backlog_check.setObjectName(u"backlog_check")

        self.tags_layout.addWidget(self.backlog_check, 2, 0, 1, 2)

        self.custom2_check = QCheckBox(self.tags_group)
        self.custom2_check.setObjectName(u"custom2_check")
        self.custom2_check.setText(u"")

        self.tags_layout.addWidget(self.custom2_check, 5, 0, 1, 1)

        self.custom2_edit = QLineEdit(self.tags_group)
        self.custom2_edit.setObjectName(u"custom2_edit")

        self.tags_layout.addWidget(self.custom2_edit, 5, 1, 1, 1)


        self.left_layout.addWidget(self.tags_group)


        self.main_layout.addLayout(self.left_layout)

        self.right_layout = QVBoxLayout()
        self.right_layout.setObjectName(u"right_layout")
        self.details_layout = QFormLayout()
        self.details_layout.setObjectName(u"details_layout")
        self.details_layout.setSizeConstraint(QLayout.SetFixedSize)
        self.details_layout.setFieldGrowthPolicy(QFormLayout.AllNonFixedFieldsGrow)
        self.details_layout.setLabelAlignment(Qt.AlignmentFlag.AlignRight|Qt.AlignTrailing|Qt.AlignmentFlag.AlignVCenter)
        self.details_layout.setHorizontalSpacing(12)
        self.details_layout.setVerticalSpacing(12)
        self.details_layout.setContentsMargins(6, 6, 6, 6)
        self.lbl_dev = QLabel(GameDetails)
        self.lbl_dev.setObjectName(u"lbl_dev")
        sizePolicy1 = QSizePolicy(QSizePolicy.Policy.Fixed, QSizePolicy.Policy.Preferred)
        sizePolicy1.setHorizontalStretch(0)
        sizePolicy1.setVerticalStretch(0)
        sizePolicy1.setHeightForWidth(self.lbl_dev.sizePolicy().hasHeightForWidth())
        self.lbl_dev.setSizePolicy(sizePolicy1)
        font = QFont()
        font.setBold(True)
        self.lbl_dev.setFont(font)
        self.lbl_dev.setAlignment(Qt.AlignmentFlag.AlignRight|Qt.AlignTrailing|Qt.AlignmentFlag.AlignVCenter)

        self.details_layout.setWidget(0, QFormLayout.LabelRole, self.lbl_dev)

        self.dev = QLabel(GameDetails)
        self.dev.setObjectName(u"dev")
        self.dev.setText(u"error")
        self.dev.setTextInteractionFlags(Qt.LinksAccessibleByMouse|Qt.TextSelectableByMouse)

        self.details_layout.setWidget(0, QFormLayout.ItemRole.FieldRole, self.dev)

        self.lbl_app_name = QLabel(GameDetails)
        self.lbl_app_name.setObjectName(u"lbl_app_name")
        sizePolicy1.setHeightForWidth(self.lbl_app_name.sizePolicy().hasHeightForWidth())
        self.lbl_app_name.setSizePolicy(sizePolicy1)
        self.lbl_app_name.setFont(font)
        self.lbl_app_name.setAlignment(Qt.AlignmentFlag.AlignRight|Qt.AlignTrailing|Qt.AlignmentFlag.AlignVCenter)

        self.details_layout.setWidget(1, QFormLayout.LabelRole, self.lbl_app_name)

        self.app_name = QLabel(GameDetails)
        self.app_name.setObjectName(u"app_name")
        self.app_name.setText(u"error")
        self.app_name.setTextInteractionFlags(Qt.LinksAccessibleByMouse|Qt.TextSelectableByMouse)

        self.details_layout.setWidget(1, QFormLayout.ItemRole.FieldRole, self.app_name)

        self.lbl_version = QLabel(GameDetails)
        self.lbl_version.setObjectName(u"lbl_version")
        sizePolicy1.setHeightForWidth(self.lbl_version.sizePolicy().hasHeightForWidth())
        self.lbl_version.setSizePolicy(sizePolicy1)
        self.lbl_version.setFont(font)
        self.lbl_version.setAlignment(Qt.AlignmentFlag.AlignRight|Qt.AlignTrailing|Qt.AlignmentFlag.AlignVCenter)

        self.details_layout.setWidget(2, QFormLayout.LabelRole, self.lbl_version)

        self.version = QLabel(GameDetails)
        self.version.setObjectName(u"version")
        self.version.setText(u"error")
        self.version.setTextInteractionFlags(Qt.LinksAccessibleByMouse|Qt.TextSelectableByMouse)

        self.details_layout.setWidget(2, QFormLayout.ItemRole.FieldRole, self.version)

        self.lbl_grade = QLabel(GameDetails)
        self.lbl_grade.setObjectName(u"lbl_grade")
        sizePolicy1.setHeightForWidth(self.lbl_grade.sizePolicy().hasHeightForWidth())
        self.lbl_grade.setSizePolicy(sizePolicy1)
        self.lbl_grade.setFont(font)
        self.lbl_grade.setAlignment(Qt.AlignmentFlag.AlignRight|Qt.AlignTrailing|Qt.AlignmentFlag.AlignVCenter)

        self.details_layout.setWidget(3, QFormLayout.LabelRole, self.lbl_grade)

        self.grade = QLabel(GameDetails)
        self.grade.setObjectName(u"grade")
        self.grade.setText(u"error")
        self.grade.setOpenExternalLinks(True)
        self.grade.setTextInteractionFlags(Qt.LinksAccessibleByMouse|Qt.TextSelectableByMouse)

        self.details_layout.setWidget(3, QFormLayout.ItemRole.FieldRole, self.grade)

        self.lbl_install_size = QLabel(GameDetails)
        self.lbl_install_size.setObjectName(u"lbl_install_size")
        sizePolicy1.setHeightForWidth(self.lbl_install_size.sizePolicy().hasHeightForWidth())
        self.lbl_install_size.setSizePolicy(sizePolicy1)
        self.lbl_install_size.setFont(font)
        self.lbl_install_size.setAlignment(Qt.AlignmentFlag.AlignRight|Qt.AlignTrailing|Qt.AlignmentFlag.AlignVCenter)

        self.details_layout.setWidget(4, QFormLayout.LabelRole, self.lbl_install_size)

        self.install_size = QLabel(GameDetails)
        self.install_size.setObjectName(u"install_size")
        self.install_size.setText(u"error")
        self.install_size.setTextInteractionFlags(Qt.LinksAccessibleByMouse|Qt.TextSelectableByMouse)

        self.details_layout.setWidget(4, QFormLayout.ItemRole.FieldRole, self.install_size)

        self.lbl_install_path = QLabel(GameDetails)
        self.lbl_install_path.setObjectName(u"lbl_install_path")
        sizePolicy1.setHeightForWidth(self.lbl_install_path.sizePolicy().hasHeightForWidth())
        self.lbl_install_path.setSizePolicy(sizePolicy1)
        self.lbl_install_path.setFont(font)
        self.lbl_install_path.setAlignment(Qt.AlignmentFlag.AlignRight|Qt.AlignTrailing|Qt.AlignmentFlag.AlignVCenter)

        self.details_layout.setWidget(5, QFormLayout.LabelRole, self.lbl_install_path)

        self.install_path = QLabel(GameDetails)
        self.install_path.setObjectName(u"install_path")
        self.install_path.setText(u"error")
        self.install_path.setWordWrap(True)
        self.install_path.setTextInteractionFlags(Qt.LinksAccessibleByMouse|Qt.TextSelectableByMouse)

        self.details_layout.setWidget(5, QFormLayout.ItemRole.FieldRole, self.install_path)

        self.lbl_platform = QLabel(GameDetails)
        self.lbl_platform.setObjectName(u"lbl_platform")
        sizePolicy1.setHeightForWidth(self.lbl_platform.sizePolicy().hasHeightForWidth())
        self.lbl_platform.setSizePolicy(sizePolicy1)
        self.lbl_platform.setFont(font)
        self.lbl_platform.setAlignment(Qt.AlignmentFlag.AlignRight|Qt.AlignTrailing|Qt.AlignmentFlag.AlignVCenter)

        self.details_layout.setWidget(6, QFormLayout.LabelRole, self.lbl_platform)

        self.platform = QLabel(GameDetails)
        self.platform.setObjectName(u"platform")
        self.platform.setText(u"error")

        self.details_layout.setWidget(6, QFormLayout.ItemRole.FieldRole, self.platform)

        self.lbl_game_actions = QLabel(GameDetails)
        self.lbl_game_actions.setObjectName(u"lbl_game_actions")
        sizePolicy1.setHeightForWidth(self.lbl_game_actions.sizePolicy().hasHeightForWidth())
        self.lbl_game_actions.setSizePolicy(sizePolicy1)
        self.lbl_game_actions.setFont(font)
        self.lbl_game_actions.setAlignment(Qt.AlignmentFlag.AlignRight|Qt.AlignTrailing|Qt.AlignmentFlag.AlignVCenter)

        self.details_layout.setWidget(7, QFormLayout.LabelRole, self.lbl_game_actions)

        self.game_actions_stack = QStackedWidget(GameDetails)
        self.game_actions_stack.setObjectName(u"game_actions_stack")
        sizePolicy.setHeightForWidth(self.game_actions_stack.sizePolicy().hasHeightForWidth())
        self.game_actions_stack.setSizePolicy(sizePolicy)
        self.game_actions_stack.setMinimumSize(QSize(250, 0))
        self.game_actions_stack.setMaximumSize(QSize(250, 16777215))
        self.installed_page = QWidget()
        self.installed_page.setObjectName(u"installed_page")
        self.installed_layout = QVBoxLayout(self.installed_page)
        self.installed_layout.setObjectName(u"installed_layout")
        self.installed_layout.setContentsMargins(0, 0, 0, 0)
        self.modify_button = QPushButton(self.installed_page)
        self.modify_button.setObjectName(u"modify_button")

        self.installed_layout.addWidget(self.modify_button)

        self.verify_stack = QStackedWidget(self.installed_page)
        self.verify_stack.setObjectName(u"verify_stack")
        self.verify_button_page = QWidget()
        self.verify_button_page.setObjectName(u"verify_button_page")
        self.verify_page_layout = QHBoxLayout(self.verify_button_page)
        self.verify_page_layout.setSpacing(0)
        self.verify_page_layout.setObjectName(u"verify_page_layout")
        self.verify_page_layout.setContentsMargins(0, 0, 0, 0)
        self.verify_button = QPushButton(self.verify_button_page)
        self.verify_button.setObjectName(u"verify_button")

        self.verify_page_layout.addWidget(self.verify_button)

        self.verify_stack.addWidget(self.verify_button_page)
        self.verify_progress_page = QWidget()
        self.verify_progress_page.setObjectName(u"verify_progress_page")
        self.verify_progress_layout = QHBoxLayout(self.verify_progress_page)
        self.verify_progress_layout.setSpacing(0)
        self.verify_progress_layout.setObjectName(u"verify_progress_layout")
        self.verify_progress_layout.setContentsMargins(0, 0, 0, 0)
        self.verify_progress = QProgressBar(self.verify_progress_page)
        self.verify_progress.setObjectName(u"verify_progress")
        self.verify_progress.setValue(24)

        self.verify_progress_layout.addWidget(self.verify_progress)

        self.verify_stack.addWidget(self.verify_progress_page)

        self.installed_layout.addWidget(self.verify_stack)

        self.repair_button = QPushButton(self.installed_page)
        self.repair_button.setObjectName(u"repair_button")

        self.installed_layout.addWidget(self.repair_button)

        self.move_stack = QStackedWidget(self.installed_page)
        self.move_stack.setObjectName(u"move_stack")
        self.move_button_page = QWidget()
        self.move_button_page.setObjectName(u"move_button_page")
        self.move_button_layout = QHBoxLayout(self.move_button_page)
        self.move_button_layout.setSpacing(0)
        self.move_button_layout.setObjectName(u"move_button_layout")
        self.move_button_layout.setContentsMargins(0, 0, 0, 0)
        self.move_button = QPushButton(self.move_button_page)
        self.move_button.setObjectName(u"move_button")
        sizePolicy2 = QSizePolicy(QSizePolicy.Policy.Minimum, QSizePolicy.Policy.Fixed)
        sizePolicy2.setHorizontalStretch(0)
        sizePolicy2.setVerticalStretch(0)
        sizePolicy2.setHeightForWidth(self.move_button.sizePolicy().hasHeightForWidth())
        self.move_button.setSizePolicy(sizePolicy2)

        self.move_button_layout.addWidget(self.move_button)

        self.move_stack.addWidget(self.move_button_page)
        self.move_progress_page = QWidget()
        self.move_progress_page.setObjectName(u"move_progress_page")
        self.move_progress_layout = QHBoxLayout(self.move_progress_page)
        self.move_progress_layout.setSpacing(0)
        self.move_progress_layout.setObjectName(u"move_progress_layout")
        self.move_progress_layout.setContentsMargins(0, 0, 0, 0)
        self.move_progress = QProgressBar(self.move_progress_page)
        self.move_progress.setObjectName(u"move_progress")
        self.move_progress.setValue(24)

        self.move_progress_layout.addWidget(self.move_progress)

        self.move_stack.addWidget(self.move_progress_page)

        self.installed_layout.addWidget(self.move_stack)

        self.uninstall_button = QPushButton(self.installed_page)
        self.uninstall_button.setObjectName(u"uninstall_button")

        self.installed_layout.addWidget(self.uninstall_button)

        self.game_actions_stack.addWidget(self.installed_page)
        self.uninstalled_page = QWidget()
        self.uninstalled_page.setObjectName(u"uninstalled_page")
        self.uninstalled_layout = QVBoxLayout(self.uninstalled_page)
        self.uninstalled_layout.setObjectName(u"uninstalled_layout")
        self.uninstalled_layout.setContentsMargins(0, 0, 0, 0)
        self.install_button = QPushButton(self.uninstalled_page)
        self.install_button.setObjectName(u"install_button")

        self.uninstalled_layout.addWidget(self.install_button)

        self.import_button = QPushButton(self.uninstalled_page)
        self.import_button.setObjectName(u"import_button")

        self.uninstalled_layout.addWidget(self.import_button)

        self.uninstalled_page_bot_vspacer = QSpacerItem(0, 0, QSizePolicy.Policy.Minimum, QSizePolicy.Policy.Expanding)

        self.uninstalled_layout.addItem(self.uninstalled_page_bot_vspacer)

        self.game_actions_stack.addWidget(self.uninstalled_page)

        self.details_layout.setWidget(7, QFormLayout.ItemRole.FieldRole, self.game_actions_stack)


        self.right_layout.addLayout(self.details_layout)

        self.requirements_group = QFrame(GameDetails)
        self.requirements_group.setObjectName(u"requirements_group")
        sizePolicy3 = QSizePolicy(QSizePolicy.Policy.Expanding, QSizePolicy.Policy.Fixed)
        sizePolicy3.setHorizontalStretch(0)
        sizePolicy3.setVerticalStretch(0)
        sizePolicy3.setHeightForWidth(self.requirements_group.sizePolicy().hasHeightForWidth())
        self.requirements_group.setSizePolicy(sizePolicy3)
        self.requirements_group.setFrameShape(QFrame.Shape.StyledPanel)
        self.requirements_group.setFrameShadow(QFrame.Sunken)
        self.requirements_layout = QHBoxLayout(self.requirements_group)
        self.requirements_layout.setObjectName(u"requirements_layout")
        self.requirements_layout.setContentsMargins(0, 0, 0, 0)

        self.right_layout.addWidget(self.requirements_group)


        self.main_layout.addLayout(self.right_layout)

        self.main_layout.setStretch(1, 1)

        self.retranslateUi(GameDetails)

        self.game_actions_stack.setCurrentIndex(0)
        self.verify_stack.setCurrentIndex(0)
        self.move_stack.setCurrentIndex(0)

    # setupUi

    def retranslateUi(self, GameDetails):
        self.tags_group.setTitle(QCoreApplication.translate("GameDetails", u"Tags", None))
        self.completed_check.setText(QCoreApplication.translate("GameDetails", u"Completed", None))
        self.hidden_check.setText(QCoreApplication.translate("GameDetails", u"Hidden", None))
        self.favorites_check.setText(QCoreApplication.translate("GameDetails", u"Favorites", None))
        self.backlog_check.setText(QCoreApplication.translate("GameDetails", u"Backlog", None))
        self.lbl_dev.setText(QCoreApplication.translate("GameDetails", u"Developer", None))
        self.lbl_app_name.setText(QCoreApplication.translate("GameDetails", u"Application name", None))
        self.lbl_version.setText(QCoreApplication.translate("GameDetails", u"Version", None))
        self.lbl_grade.setText(QCoreApplication.translate("GameDetails", u"ProtonDB grade", None))
        self.lbl_install_size.setText(QCoreApplication.translate("GameDetails", u"Installation size", None))
        self.lbl_install_path.setText(QCoreApplication.translate("GameDetails", u"Installation path", None))
        self.lbl_platform.setText(QCoreApplication.translate("GameDetails", u"Platform", None))
        self.lbl_game_actions.setText(QCoreApplication.translate("GameDetails", u"Actions", None))
        self.modify_button.setText(QCoreApplication.translate("GameDetails", u"Modify", None))
        self.verify_button.setText(QCoreApplication.translate("GameDetails", u"Verify", None))
        self.repair_button.setText(QCoreApplication.translate("GameDetails", u"Repair", None))
        self.move_button.setText(QCoreApplication.translate("GameDetails", u"Move", None))
        self.uninstall_button.setText(QCoreApplication.translate("GameDetails", u"Uninstall", None))
        self.install_button.setText(QCoreApplication.translate("GameDetails", u"Install", None))
        self.import_button.setText(QCoreApplication.translate("GameDetails", u"Import", None))
        pass
    # retranslateUi




from enum import IntEnum
from typing import List

from PySide6.QtCore import Signal, Qt, Slot
from PySide6.QtGui import QShowEvent
from PySide6.QtWidgets import QMessageBox, QWidget, QSizePolicy

from rare.ui.components.tabs.store.wishlist import Ui_Wishlist
from rare.utils.misc import qta_icon
from rare.widgets.flow_layout import FlowLayout
from rare.widgets.side_tab import SideTabContents
from rare.widgets.sliding_stack import SlidingStackedWidget
from .api.models.response import WishlistItemModel, CatalogOfferModel
from .store_api import StoreAPI
from .widgets.details import StoreDetailsWidget
from .widgets.items import WishlistItemWidget


class WishlistPage(SlidingStackedWidget, SideTabContents):
    def __init__(self, api: StoreAPI, parent=None):
        super(WishlistPage, self).__init__(parent=parent)
        self.implements_scrollarea = True

        self.wishlist_widget = WishlistWidget(api, parent=self)
        self.wishlist_widget.setSizePolicy(QSizePolicy.Policy.Expanding, QSizePolicy.Policy.Fixed)
        self.wishlist_widget.set_title.connect(self.set_title)
        self.wishlist_widget.show_details.connect(self.show_details)

        self.details_widget = StoreDetailsWidget([], api, parent=self)
        self.details_widget.setSizePolicy(QSizePolicy.Policy.Expanding, QSizePolicy.Policy.Fixed)
        self.details_widget.set_title.connect(self.set_title)
        self.details_widget.back_clicked.connect(self.show_main)

        self.setDirection(Qt.Orientation.Horizontal)
        self.addWidget(self.wishlist_widget)
        self.addWidget(self.details_widget)

    @Slot()
    def show_main(self):
        self.slideInWidget(self.wishlist_widget)

    @Slot(object)
    def show_details(self, game: CatalogOfferModel):
        self.details_widget.update_game(game)
        self.slideInWidget(self.details_widget)


class WishlistOrder(IntEnum):
    NAME = 1
    PRICE = 2
    DISCOUNT = 3
    DEVELOPER = 4


class WishlistFilter(IntEnum):
    NONE = 0
    DISCOUNT = 1


class WishlistWidget(QWidget, SideTabContents):
    show_details = Signal(CatalogOfferModel)
    update_wishlist = Signal()

    def __init__(self, api: StoreAPI, parent=None):
        super(WishlistWidget, self).__init__(parent=parent)
        self.implements_scrollarea = True
        self.api = api
        self.ui = Ui_Wishlist()
        self.ui.setupUi(self)
        self.ui.main_layout.setContentsMargins(0, 0, 3, 0)

        self.wishlist_layout = FlowLayout()
        self.ui.container_layout.addLayout(self.wishlist_layout, stretch=1)

        filters = {
            WishlistFilter.NONE: self.tr("All items"),
            WishlistFilter.DISCOUNT: self.tr("Discount"),
        }
        for data, text in filters.items():
            self.ui.filter_combo.addItem(text, data)
        self.ui.filter_combo.currentIndexChanged.connect(self.filter_wishlist)

        sortings = {
            WishlistOrder.NAME: self.tr("Name"),
            WishlistOrder.PRICE: self.tr("Price"),
            WishlistOrder.DISCOUNT: self.tr("Discount"),
            WishlistOrder.DEVELOPER: self.tr("Developer"),
        }
        for data, text in sortings.items():
            self.ui.order_combo.addItem(text, data)
        self.ui.order_combo.currentIndexChanged.connect(self.order_wishlist)

        self.ui.reload_button.setIcon(qta_icon("fa.refresh", color="white"))
        self.ui.reload_button.clicked.connect(self.__update_widget)

        self.ui.reverse_check.stateChanged.connect(
            lambda: self.order_wishlist(self.ui.order_combo.currentIndex())
        )

        self.setEnabled(False)

    def showEvent(self, a0: QShowEvent) -> None:
        self.__update_widget()
        return super().showEvent(a0)

    def __update_widget(self):
        self.setEnabled(False)
        self.api.get_wishlist(self.set_wishlist)

    def delete_from_wishlist(self, game: CatalogOfferModel):
        self.api.remove_from_wishlist(
            game.namespace,
            game.id,
            lambda success: self.__update_widget()
            if success
            else QMessageBox.warning(
                self, "Error", self.tr("Could not remove game from wishlist")
            ),
        )
        self.update_wishlist.emit()

    @Slot(int)
    def filter_wishlist(self, index: int = int(WishlistFilter.NONE)):
        list_filter = self.ui.filter_combo.itemData(index, Qt.ItemDataRole.UserRole)
        widgets = self.ui.container.findChildren(WishlistItemWidget, options=Qt.FindChildOption.FindDirectChildrenOnly)
        for w in widgets:
            if list_filter == WishlistFilter.NONE:
                w.setVisible(True)
            elif list_filter == WishlistFilter.DISCOUNT:
                w.setVisible(bool(w.catalog_game.price.totalPrice.discount))
            else:
                w.setVisible(True)
        have_visible = any(map(lambda x: x.isVisible(), widgets))
        self.ui.no_games_label.setVisible(not have_visible)

    @Slot(int)
    def order_wishlist(self, index: int = int(WishlistOrder.NAME)):
        list_order = self.ui.order_combo.itemData(index, Qt.ItemDataRole.UserRole)
        widgets = self.ui.container.findChildren(WishlistItemWidget, options=Qt.FindChildOption.FindDirectChildrenOnly)
        for w in widgets:
            self.wishlist_layout.removeWidget(w)

        if list_order == WishlistOrder.NAME:
            def func(x: WishlistItemWidget):
                return x.catalog_game.title
        elif list_order == WishlistOrder.PRICE:
            def func(x: WishlistItemWidget):
                return x.catalog_game.price.totalPrice.discountPrice
        elif list_order == WishlistOrder.DEVELOPER:
            def func(x: WishlistItemWidget):
                return x.catalog_game.seller["name"]
        elif list_order == WishlistOrder.DISCOUNT:
            def func(x: WishlistItemWidget):
                discount = x.catalog_game.price.totalPrice.discountPrice
                original = x.catalog_game.price.totalPrice.originalPrice
                return 1 - (discount / original)
        else:
            def func(x: WishlistItemWidget):
                return x.catalog_game.title

        reverse = self.ui.reverse_check.isChecked()
        widgets = sorted(widgets, key=func, reverse=reverse)
        for w in widgets:
            self.wishlist_layout.addWidget(w)

    def set_wishlist(self, wishlist: List[WishlistItemModel] = None):
        if wishlist and wishlist[0] == "error":
            return

        widgets = self.ui.container.findChildren(WishlistItemWidget, options=Qt.FindChildOption.FindDirectChildrenOnly)
        for w in widgets:
            self.wishlist_layout.removeWidget(w)
            w.deleteLater()

        self.ui.no_games_label.setVisible(bool(wishlist))

        for game in wishlist:
            w = WishlistItemWidget(self.api.cached_manager, game.offer, self.ui.container)
            w.show_details.connect(self.show_details)
            w.delete_from_wishlist.connect(self.delete_from_wishlist)
            self.wishlist_layout.addWidget(w)

        self.order_wishlist(self.ui.order_combo.currentIndex())
        self.filter_wishlist(self.ui.filter_combo.currentIndex())

        self.setEnabled(True)



from PySide6.QtWidgets import QGroupBox, QLayout

from rare.widgets.loading_widget import LoadingWidget


class StoreGroup(QGroupBox):
    def __init__(self, title: str, layout: type[QLayout], parent=None):
        super().__init__(parent=parent)
        self.setTitle(title)
        self.main_layout = layout(self)
        self.loading_widget = LoadingWidget(autostart=True, parent=self)

    def loading(self, state: bool) -> None:
        if state:
            self.loading_widget.start()
        else:
            self.loading_widget.stop()




import sys
from argparse import Namespace

from legendary.utils import webview_login


def launch(args: Namespace) -> int:
    if webview_login.do_webview_login(
            callback_code=sys.stdout.write, user_agent=f'EpicGamesLauncher/{args.egl_version}'
    ):
        return 0
    else:
        return 1



# -*- coding: utf-8 -*-

################################################################################
## Form generated from reading UI file 'cloud_sync_widget.ui'
##
## Created by: Qt User Interface Compiler version 6.7.2
##
## WARNING! All changes made in this file will be lost when recompiling UI file!
################################################################################

from PySide6.QtCore import (QCoreApplication, QDate, QDateTime, QLocale,
    QMetaObject, QObject, QPoint, QRect,
    QSize, QTime, QUrl, Qt)
from PySide6.QtGui import (QBrush, QColor, QConicalGradient, QCursor,
    QFont, QFontDatabase, QGradient, QIcon,
    QImage, QKeySequence, QLinearGradient, QPainter,
    QPalette, QPixmap, QRadialGradient, QTransform)
from PySide6.QtWidgets import (QApplication, QGroupBox, QHBoxLayout, QLabel,
    QPushButton, QSizePolicy, QVBoxLayout, QWidget)

class Ui_CloudSyncWidget(object):
    def setupUi(self, CloudSyncWidget):
        if not CloudSyncWidget.objectName():
            CloudSyncWidget.setObjectName(u"CloudSyncWidget")
        CloudSyncWidget.resize(438, 137)
        sizePolicy = QSizePolicy(QSizePolicy.Policy.Preferred, QSizePolicy.Policy.Fixed)
        sizePolicy.setHorizontalStretch(0)
        sizePolicy.setVerticalStretch(0)
        sizePolicy.setHeightForWidth(CloudSyncWidget.sizePolicy().hasHeightForWidth())
        CloudSyncWidget.setSizePolicy(sizePolicy)
        CloudSyncWidget.setWindowTitle(u"SyncWidget")
        self.main_layout = QHBoxLayout(CloudSyncWidget)
        self.main_layout.setObjectName(u"main_layout")
        self.main_layout.setContentsMargins(0, 0, 0, 0)
        self.local_group = QGroupBox(CloudSyncWidget)
        self.local_group.setObjectName(u"local_group")
        self.local_layout = QVBoxLayout(self.local_group)
        self.local_layout.setObjectName(u"local_layout")
        self.date_info_local = QLabel(self.local_group)
        self.date_info_local.setObjectName(u"date_info_local")
        self.date_info_local.setText(u"")
        self.date_info_local.setAlignment(Qt.AlignCenter)

        self.local_layout.addWidget(self.date_info_local)

        self.icon_local = QLabel(self.local_group)
        self.icon_local.setObjectName(u"icon_local")
        sizePolicy1 = QSizePolicy(QSizePolicy.Policy.Expanding, QSizePolicy.Policy.Preferred)
        sizePolicy1.setHorizontalStretch(0)
        sizePolicy1.setVerticalStretch(0)
        sizePolicy1.setHeightForWidth(self.icon_local.sizePolicy().hasHeightForWidth())
        self.icon_local.setSizePolicy(sizePolicy1)
        self.icon_local.setText(u"")
        self.icon_local.setAlignment(Qt.AlignCenter)

        self.local_layout.addWidget(self.icon_local)

        self.age_label_local = QLabel(self.local_group)
        self.age_label_local.setObjectName(u"age_label_local")
        self.age_label_local.setAlignment(Qt.AlignCenter)

        self.local_layout.addWidget(self.age_label_local)

        self.upload_button = QPushButton(self.local_group)
        self.upload_button.setObjectName(u"upload_button")
        self.upload_button.setMinimumSize(QSize(192, 0))

        self.local_layout.addWidget(self.upload_button)


        self.main_layout.addWidget(self.local_group)

        self.cloud_group = QGroupBox(CloudSyncWidget)
        self.cloud_group.setObjectName(u"cloud_group")
        self.cloud_layout = QVBoxLayout(self.cloud_group)
        self.cloud_layout.setObjectName(u"cloud_layout")
        self.date_info_remote = QLabel(self.cloud_group)
        self.date_info_remote.setObjectName(u"date_info_remote")
        self.date_info_remote.setText(u"")
        self.date_info_remote.setAlignment(Qt.AlignCenter)

        self.cloud_layout.addWidget(self.date_info_remote)

        self.icon_remote = QLabel(self.cloud_group)
        self.icon_remote.setObjectName(u"icon_remote")
        sizePolicy1.setHeightForWidth(self.icon_remote.sizePolicy().hasHeightForWidth())
        self.icon_remote.setSizePolicy(sizePolicy1)
        self.icon_remote.setText(u"")
        self.icon_remote.setAlignment(Qt.AlignCenter)

        self.cloud_layout.addWidget(self.icon_remote)

        self.age_label_remote = QLabel(self.cloud_group)
        self.age_label_remote.setObjectName(u"age_label_remote")
        self.age_label_remote.setText(u"")
        self.age_label_remote.setAlignment(Qt.AlignCenter)

        self.cloud_layout.addWidget(self.age_label_remote)

        self.download_button = QPushButton(self.cloud_group)
        self.download_button.setObjectName(u"download_button")
        self.download_button.setMinimumSize(QSize(192, 0))

        self.cloud_layout.addWidget(self.download_button)


        self.main_layout.addWidget(self.cloud_group)


        self.retranslateUi(CloudSyncWidget)
    # setupUi

    def retranslateUi(self, CloudSyncWidget):
        self.local_group.setTitle(QCoreApplication.translate("CloudSyncWidget", u"Local", None))
        self.age_label_local.setText("")
        self.upload_button.setText(QCoreApplication.translate("CloudSyncWidget", u"Upload", None))
        self.cloud_group.setTitle(QCoreApplication.translate("CloudSyncWidget", u"Cloud", None))
        self.download_button.setText(QCoreApplication.translate("CloudSyncWidget", u"Download", None))
        pass
    # retranslateUi




import platform
from argparse import Namespace
from enum import IntEnum
from logging import getLogger

from PySide6.QtCore import QObject, Signal, QSettings
from requests.exceptions import HTTPError, ConnectionError

from rare.lgndr.core import LegendaryCore
from rare.utils.metrics import timelogger
from rare.models.options import options
from .worker import Worker

logger = getLogger("FetchWorker")


class FetchWorker(Worker):
    class Result(IntEnum):
        ERROR = 0
        GAMESDLCS = 1
        ENTITLEMENTS = 2

    class Signals(QObject):
        progress = Signal(int, str)
        result = Signal(object, int)

    def __init__(self, core: LegendaryCore, args: Namespace):
        super(Worker, self).__init__()
        self.signals = FetchWorker.Signals()
        self.core = core
        self.args = args
        self.settings = QSettings()


class EntitlementsWorker(FetchWorker):

    def run_real(self):

        want_entitlements = not self.settings.value(*options.exclude_entitlements)

        entitlements = ()
        if want_entitlements and not self.args.offline:
            # Get entitlements, Ubisoft integration also uses them
            self.signals.progress.emit(0, self.signals.tr("Updating entitlements"))
            with timelogger(logger, "Request entitlements"):
                try:
                    entitlements = self.core.egs.get_user_entitlements_full()
                except AttributeError as e:
                    logger.warning(e)
                    entitlements = self.core.egs.get_user_entitlements()
            self.core.lgd.entitlements = entitlements
            logger.info(f"Entitlements: %s", len(list(entitlements)))
        self.signals.result.emit(entitlements, FetchWorker.Result.ENTITLEMENTS)
        return


class GamesDlcsWorker(FetchWorker):

    def run_real(self):

        # Fetch regular EGL games with assets
        # want_unreal = self.settings.value(*options.unreal_meta) or self.args.debug
        # want_win32 = self.settings.value(*options.win32_meta) or self.args.debug
        # want_macos = self.settings.value(*options.macos_meta) or self.args.debug
        want_unreal = self.settings.value(*options.unreal_meta)
        want_win32 = self.settings.value(*options.win32_meta)
        want_macos = self.settings.value(*options.macos_meta)
        want_non_asset = not self.settings.value(*options.exclude_non_asset)
        need_macos = platform.system() == "Darwin"
        need_windows = not any([want_win32, want_macos, need_macos]) and not self.args.offline

        if want_win32:
            logger.info(
                "Requesting Win32 metadata due to %s, %s Unreal engine",
                "settings" if want_win32 else "debug",
                "with" if want_unreal else "without"
            )
            self.signals.progress.emit(00, self.signals.tr("Updating game metadata for Windows"))
            with timelogger(logger, "Request Win32 games"):
                self.core.get_game_and_dlc_list(
                    update_assets=not self.args.offline, platform="Win32", skip_ue=not want_unreal
                )

        if need_macos or want_macos:
            logger.info(
                "Requesting macOS metadata due to %s, %s Unreal engine",
                "platform" if need_macos else "settings" if want_macos else "debug",
                "with" if want_unreal else "without"
            )
            self.signals.progress.emit(15, self.signals.tr("Updating game metadata for macOS"))
            with timelogger(logger, "Request macOS games"):
                self.core.get_game_and_dlc_list(
                    update_assets=not self.args.offline, platform="Mac", skip_ue=not want_unreal
                )

        if need_windows:
            self.signals.progress.emit(00, self.signals.tr("Updating game metadata for Windows"))
            logger.info(
                "Requesting Windows metadata, %s Unreal engine",
                "with" if want_unreal else "without"
            )
        with timelogger(logger, "Request Windows games"):
            games, dlc_dict = self.core.get_game_and_dlc_list(
                update_assets=need_windows, platform="Windows", skip_ue=not want_unreal
            )
        logger.info(f"Games: %s. Games with DLCs: %s", len(games), len(dlc_dict))

        # Fetch non-asset games
        if want_non_asset:
            self.signals.progress.emit(30, self.signals.tr("Updating non-asset game metadata"))
            try:
                with timelogger(logger, "Request non-asset"):
                    na_games, na_dlc_dict = self.core.get_non_asset_library_items(force_refresh=False, skip_ue=False)
            except (HTTPError, ConnectionError) as e:
                logger.error(f"Network error while fetching non asset games")
                logger.error(e)
                na_games, na_dlc_dict = ([], {})
            # NOTE: This is here because of broken appIds from Epic
            # https://discord.com/channels/826881530310819914/884510635642216499/1111321692703305729
            except Exception as e:
                logger.error("General exception while fetching non asset games from EGS.")
                logger.error(e)
                na_games, na_dlc_dict = ([], {})
            logger.info("Non-asset: %s. Non-asset with DLCs: %s", len(na_games), len(na_dlc_dict))

            # Combine the two games lists and the two dlc dictionaries between regular and non-asset results
            games += na_games
            for catalog_id, dlcs in na_dlc_dict.items():
                if catalog_id in dlc_dict.keys():
                    dlc_dict[catalog_id] += dlcs
                else:
                    dlc_dict[catalog_id] = dlcs
            logger.info(f"Games: {len(games)}. Games with DLCs: {len(dlc_dict)}")

        self.signals.progress.emit(40, self.signals.tr("Preparing library"))
        self.signals.result.emit((games, dlc_dict), FetchWorker.Result.GAMESDLCS)



from rare._version import __version__, __version_tuple__

__codename__ = "Boga Discus"

# For PyCharm profiler
if __name__ == "__main__":
    import sys
    from rare.main import main

    sys.exit(main())

__all__ = ["__version__", "__version_tuple__", "__codename__"]



import os.path
import platform as pf
from logging import getLogger
from typing import Tuple

from PySide6.QtCore import Qt, Slot
from PySide6.QtGui import QShowEvent
from PySide6.QtWidgets import QFileDialog, QComboBox, QLineEdit
from legendary.models.game import Game, InstalledGame

from rare.components.tabs.settings.widgets.env_vars import EnvVars
from rare.components.tabs.settings.widgets.game import GameSettingsBase
from rare.components.tabs.settings.widgets.launch import LaunchSettingsBase
from rare.components.tabs.settings.widgets.overlay import DxvkSettings
from rare.components.tabs.settings.widgets.wrappers import WrapperSettings
from rare.models.game import RareGame
from rare.utils import config_helper as config
from rare.widgets.indicator_edit import PathEdit, IndicatorReasonsCommon

if pf.system() != "Windows":
    from rare.components.tabs.settings.widgets.wine import WineSettings
    if pf.system() in {"Linux", "FreeBSD"}:
        from rare.components.tabs.settings.widgets.proton import ProtonSettings
        from rare.components.tabs.settings.widgets.overlay import MangoHudSettings

logger = getLogger("GameSettings")


class GameWrapperSettings(WrapperSettings):
    def __init__(self, parent=None):
        super().__init__(parent=parent)

    def load_settings(self, app_name: str):
        self.app_name = app_name


class GameLaunchSettings(LaunchSettingsBase):

    def __init__(self, parent=None):
        super(GameLaunchSettings, self).__init__(GameWrapperSettings, parent=parent)

        self.game: Game = None
        self.igame: InstalledGame = None

        self.skip_update_combo = QComboBox(self)
        self.skip_update_combo.addItem(self.tr("Default"), None)
        self.skip_update_combo.addItem(self.tr("No"), "false")
        self.skip_update_combo.addItem(self.tr("Yes"), "true")
        self.skip_update_combo.currentIndexChanged.connect(self.__skip_update_changed)

        self.offline_combo = QComboBox(self)
        self.offline_combo.addItem(self.tr("Default"), None)
        self.offline_combo.addItem(self.tr("No"), "false")
        self.offline_combo.addItem(self.tr("Yes"), "true")
        self.offline_combo.currentIndexChanged.connect(self.__offline_changed)

        self.override_exe_name_filters: Tuple[str, ...] = (
            "*.exe", "*.app", "*.bat", "*.ps1", "*.sh"
        )

        self.override_exe_edit = PathEdit(
            file_mode=QFileDialog.FileMode.ExistingFile,
            name_filters=self.override_exe_name_filters,
            placeholder=self.tr("Relative path to the replacement executable"),
            edit_func=self.__override_exe_edit_callback,
            save_func=self.__override_exe_save_callback,
            parent=self
        )

        self.launch_params_edit = QLineEdit(self)
        self.launch_params_edit.setPlaceholderText(self.tr("Game specific command line arguments"))
        self.launch_params_edit.textChanged.connect(self.__launch_params_changed)

        self.main_layout.insertRow(0, self.tr("Skip update check"), self.skip_update_combo)
        self.main_layout.insertRow(1, self.tr("Offline mode"), self.offline_combo)
        self.main_layout.insertRow(2, self.tr("Launch parameters"), self.launch_params_edit)
        self.main_layout.insertRow(3, self.tr("Override executable"), self.override_exe_edit)

    def showEvent(self, a0: QShowEvent):
        if a0.spontaneous():
            return super().showEvent(a0)

        skip_update = config.get_option(self.app_name, "skip_update_check", fallback=None)
        self.skip_update_combo.setCurrentIndex(self.offline_combo.findData(skip_update, Qt.ItemDataRole.UserRole))

        offline = config.get_option(self.app_name, "offline", fallback=None)
        self.offline_combo.setCurrentIndex(self.offline_combo.findData(offline, Qt.ItemDataRole.UserRole))

        if self.igame:
            self.offline_combo.setEnabled(self.igame.can_run_offline)
            self.override_exe_edit.setRootPath(self.igame.install_path)
        else:
            self.offline_combo.setEnabled(False)
            self.override_exe_edit.setRootPath(os.path.expanduser("~/"))

        launch_params = config.get_option(self.app_name, "start_params", "")
        self.launch_params_edit.setText(launch_params)

        override_exe = config.get_option(self.app_name, "override_exe", fallback="")
        self.override_exe_edit.setText(override_exe)

        return super().showEvent(a0)

    @Slot(int)
    def __skip_update_changed(self, index):
        data = self.skip_update_combo.itemData(index, Qt.ItemDataRole.UserRole)
        config.save_option(self.app_name, "skip_update_check", data)

    def __override_exe_edit_callback(self, path: str) -> Tuple[bool, str, int]:
        if not path or self.igame is None:
            return True, path, IndicatorReasonsCommon.VALID
        if not os.path.isabs(path):
            path = os.path.join(self.igame.install_path, path)
        # lk: Compare paths through python's commonpath because in windows we
        # cannot compare as strings
        # antonia disapproves of this
        if os.path.commonpath([self.igame.install_path, path]) != self.igame.install_path:
            return False, self.igame.install_path, IndicatorReasonsCommon.WRONG_PATH
        if not os.path.exists(path):
            return False, path, IndicatorReasonsCommon.WRONG_PATH

        if not path.endswith(tuple(map(lambda s: s.replace("*", ""), self.override_exe_name_filters))):
            return False, path, IndicatorReasonsCommon.WRONG_PATH
        path = os.path.relpath(path, self.igame.install_path)
        return True, path, IndicatorReasonsCommon.VALID

    def __override_exe_save_callback(self, path: str):
        config.save_option(self.app_name, "override_exe", path)

    @Slot(int)
    def __offline_changed(self, index):
        data = self.skip_update_combo.itemData(index, Qt.ItemDataRole.UserRole)
        config.save_option(self.app_name, "offline", data)

    def __launch_params_changed(self, value) -> None:
        config.save_option(self.app_name, "start_params", value)

    def load_settings(self, rgame: RareGame):
        self.game = rgame.game
        self.igame = rgame.igame
        self.app_name = rgame.app_name
        self.wrappers_widget.load_settings(rgame.app_name)


if pf.system() != "Windows":
    class GameWineSettings(WineSettings):
        def load_settings(self, app_name):
            self.app_name = app_name

    if pf.system() in {"Linux", "FreeBSD"}:
        class GameProtonSettings(ProtonSettings):
            def load_settings(self, app_name: str):
                self.app_name = app_name

        class GameMangoHudSettings(MangoHudSettings):
            def load_settings(self, app_name: str):
                self.app_name = app_name


class GameDxvkSettings(DxvkSettings):
    def load_settings(self, app_name: str):
        self.app_name = app_name


class GameEnvVars(EnvVars):
    def load_settings(self, app_name):
        self.app_name = app_name


class GameSettings(GameSettingsBase):
    def __init__(self, parent=None):
        if pf.system() != "Windows":
            if pf.system() in {"Linux", "FreeBSD"}:
                super(GameSettings, self).__init__(
                    GameLaunchSettings, GameDxvkSettings, GameEnvVars,
                    GameWineSettings, GameProtonSettings, GameMangoHudSettings,
                    parent=parent
                )
            else:
                super(GameSettings, self).__init__(
                    GameLaunchSettings, GameDxvkSettings, GameEnvVars,
                    GameWineSettings,
                    parent=parent
                )
        else:
            super(GameSettings, self).__init__(
                GameLaunchSettings, GameDxvkSettings, GameEnvVars,
                parent=parent
            )

    def load_settings(self, rgame: RareGame):
        self.set_title.emit(rgame.app_title)
        self.app_name = rgame.app_name
        self.launch.load_settings(rgame)
        if pf.system() != "Windows":
            self.wine.load_settings(rgame.app_name)
            if pf.system() in {"Linux", "FreeBSD"}:
                self.proton_tool.load_settings(rgame.app_name)
                self.mangohud.load_settings(rgame.app_name)
        self.dxvk.load_settings(rgame.app_name)
        self.env_vars.load_settings(rgame.app_name)



from PySide6.QtGui import QShowEvent, QHideEvent
from legendary.core import LegendaryCore

from rare.widgets.side_tab import SideTabWidget
from .api.models.response import CatalogOfferModel
from .landing import LandingWidget, LandingPage
from .search import SearchPage
from .store_api import StoreAPI
from .wishlist import WishlistPage


class StoreTab(SideTabWidget):

    def __init__(self, core: LegendaryCore, parent=None):
        super(StoreTab, self).__init__(parent=parent)
        self.init = False

        self.core = core
        # self.rcore = RareCore.instance()
        self.api = StoreAPI(
            self.core.egs.session.headers["Authorization"],
            self.core.language_code,
            self.core.country_code,
            []  # [i.asset_infos["Windows"].namespace for i in self.rcore.game_list if bool(i.asset_infos)]
        )

        self.landing = LandingPage(self.api, parent=self)
        self.landing_index = self.addTab(self.landing, self.tr("Store"))

        self.search = SearchPage(self.api, parent=self)
        self.search_index = self.addTab(self.search, self.tr("Search"))

        self.wishlist = WishlistPage(self.api, parent=self)
        self.wishlist_index = self.addTab(self.wishlist, self.tr("Wishlist"))

    def showEvent(self, a0: QShowEvent) -> None:
        if a0.spontaneous() or self.init:
            return super().showEvent(a0)
        self.init = True
        return super().showEvent(a0)

    def hideEvent(self, a0: QHideEvent) -> None:
        if a0.spontaneous():
            return super().hideEvent(a0)
        # TODO: Implement store unloading
        return super().hideEvent(a0)



import os
import platform as pf
import shutil
from typing import Tuple, List, Union, Optional

from PySide6.QtCore import QThreadPool, QSettings
from PySide6.QtCore import Signal, Slot
from PySide6.QtGui import QShowEvent
from PySide6.QtWidgets import QFileDialog, QCheckBox, QWidget, QFormLayout

from rare.models.game import RareGame
from rare.models.install import InstallDownloadModel, InstallQueueItemModel, InstallOptionsModel
from rare.shared.workers.install_info import InstallInfoWorker
from rare.ui.components.dialogs.install_dialog import Ui_InstallDialog
from rare.ui.components.dialogs.install_dialog_advanced import Ui_InstallDialogAdvanced
from rare.utils.misc import format_size, qta_icon
from rare.widgets.collapsible_widget import CollapsibleFrame
from rare.widgets.dialogs import ActionDialog, game_title
from rare.widgets.indicator_edit import PathEdit, IndicatorReasonsCommon
from rare.widgets.selective_widget import SelectiveWidget


class InstallDialogAdvanced(CollapsibleFrame):
    def __init__(self, parent=None):
        super(InstallDialogAdvanced, self).__init__(parent=parent)

        title = self.tr("Advanced options")
        self.setTitle(title)

        self.widget = QWidget(parent=self)
        self.ui = Ui_InstallDialogAdvanced()
        self.ui.setupUi(self.widget)
        self.setWidget(self.widget)


class InstallDialogSelective(CollapsibleFrame):
    stateChanged: Signal = Signal()

    def __init__(self, rgame: RareGame, parent=None):
        super(InstallDialogSelective, self).__init__(parent=parent)
        title = self.tr("Optional downloads")
        self.setTitle(title)
        self.setEnabled(bool(rgame.sdl_name))

        self.widget: SelectiveWidget = None
        self.rgame = rgame

    def update_list(self, platform: str):
        if self.widget is not None:
            self.widget.deleteLater()
        self.widget = SelectiveWidget(self.rgame, platform, parent=self)
        self.widget.stateChanged.connect(self.stateChanged)
        self.setWidget(self.widget)

    def install_tags(self):
        return self.widget.install_tags()


class InstallDialog(ActionDialog):
    result_ready = Signal(InstallQueueItemModel)

    def __init__(self, rgame: RareGame, options: InstallOptionsModel, parent=None):
        super(InstallDialog, self).__init__(parent=parent)

        header = self.tr("Install")
        bicon = qta_icon("ri.install-line")
        if options.repair_mode:
            header = self.tr("Repair")
            bicon = qta_icon("fa.wrench")
            if options.repair_and_update:
                header = self.tr("Repair and update")
        elif options.update:
            header = self.tr("Update")
        elif options.reset_sdl:
            header = self.tr("Modify")
            bicon = qta_icon("fa.gear")
        self.setWindowTitle(game_title(header, rgame.app_title))
        self.setSubtitle(game_title(header, rgame.app_title))

        install_widget = QWidget(self)
        self.ui = Ui_InstallDialog()
        self.ui.setupUi(install_widget)

        self.core = rgame.core
        self.rgame = rgame
        self.__options: InstallOptionsModel = options
        self.__download: Optional[InstallDownloadModel] = None
        self.__queue_item: Optional[InstallQueueItemModel] = None

        self.advanced = InstallDialogAdvanced(parent=self)
        self.ui.advanced_layout.addWidget(self.advanced)

        self.selectable = InstallDialogSelective(rgame, parent=self)
        self.selectable.stateChanged.connect(self.option_changed)
        self.ui.selectable_layout.addWidget(self.selectable)

        self.options_changed = False

        self.threadpool = QThreadPool(self)
        self.threadpool.setMaxThreadCount(1)

        if options.base_path:
            base_path = options.base_path
        elif rgame.is_installed:
            base_path = rgame.install_path
        else:
            base_path = self.core.get_default_install_dir(rgame.default_platform)

        self.install_dir_edit = PathEdit(
            path=base_path,
            file_mode=QFileDialog.FileMode.Directory,
            edit_func=self.install_dir_edit_callback,
            save_func=self.install_dir_save_callback,
            parent=self,
        )
        self.ui.main_layout.setWidget(
            self.ui.main_layout.getWidgetPosition(self.ui.install_dir_label)[0],
            QFormLayout.ItemRole.FieldRole,
            self.install_dir_edit,
        )

        self.install_dir_edit.setDisabled(rgame.is_installed)
        self.ui.install_dir_label.setDisabled(rgame.is_installed)
        self.ui.shortcut_label.setDisabled(rgame.is_installed)
        self.ui.shortcut_check.setDisabled(rgame.is_installed)
        self.ui.shortcut_check.setChecked(not rgame.is_installed and QSettings().value("create_shortcut", True, bool))

        self.error_box()

        self.ui.platform_combo.addItems(reversed(rgame.platforms))
        combo_text = rgame.igame.platform if rgame.is_installed else rgame.default_platform
        self.ui.platform_combo.setCurrentIndex(self.ui.platform_combo.findText(combo_text))
        self.ui.platform_combo.currentIndexChanged.connect(self.option_changed)
        self.ui.platform_combo.currentIndexChanged.connect(self.check_incompatible_platform)
        self.ui.platform_combo.currentIndexChanged.connect(self.reset_install_dir)
        self.ui.platform_combo.currentTextChanged.connect(self.selectable.update_list)

        self.ui.platform_label.setDisabled(rgame.is_installed)
        self.ui.platform_combo.setDisabled(rgame.is_installed)

        # if we are repairing, disable the SDL selection and open the dialog frame to be visible
        self.selectable.setDisabled(options.repair_mode and not options.repair_and_update)
        if options.repair_mode and not options.repair_and_update:
            self.selectable.click()

        self.advanced.ui.max_workers_spin.setValue(self.core.lgd.config.getint("Legendary", "max_workers", fallback=0))
        self.advanced.ui.max_workers_spin.valueChanged.connect(self.option_changed)

        self.advanced.ui.max_memory_spin.setValue(self.core.lgd.config.getint("Legendary", "max_memory", fallback=0))
        self.advanced.ui.max_memory_spin.valueChanged.connect(self.option_changed)

        self.advanced.ui.dl_optimizations_check.stateChanged.connect(self.option_changed)
        self.advanced.ui.force_download_check.stateChanged.connect(self.option_changed)
        self.advanced.ui.ignore_space_check.stateChanged.connect(self.option_changed)
        self.advanced.ui.download_only_check.stateChanged.connect(
            lambda: self.non_reload_option_changed("download_only")
        )
        self.ui.shortcut_check.stateChanged.connect(
            lambda: self.non_reload_option_changed("shortcut")
        )

        self.reset_install_dir(self.ui.platform_combo.currentIndex())
        self.selectable.update_list(self.ui.platform_combo.currentText())
        self.check_incompatible_platform(self.ui.platform_combo.currentIndex())

        self.accept_button.setEnabled(False)

        if self.__options.overlay:
            self.ui.platform_label.setEnabled(False)
            self.ui.platform_combo.setEnabled(False)
            self.advanced.ui.ignore_space_label.setEnabled(False)
            self.advanced.ui.ignore_space_check.setEnabled(False)
            self.advanced.ui.download_only_label.setEnabled(False)
            self.advanced.ui.download_only_check.setEnabled(False)
            self.ui.shortcut_label.setEnabled(False)
            self.ui.shortcut_check.setEnabled(False)
            self.selectable.setEnabled(False)

        if pf.system() == "Darwin":
            self.ui.shortcut_label.setDisabled(True)
            self.ui.shortcut_check.setDisabled(True)
            self.ui.shortcut_check.setChecked(False)
            self.ui.shortcut_check.setToolTip(self.tr("Creating a shortcut is not supported on macOS"))

        self.advanced.ui.install_prereqs_label.setEnabled(False)
        self.advanced.ui.install_prereqs_check.setEnabled(False)
        self.advanced.ui.install_prereqs_check.stateChanged.connect(
            lambda: self.non_reload_option_changed("install_prereqs")
        )

        self.non_reload_option_changed("shortcut")

        self.advanced.ui.install_prereqs_check.setChecked(self.__options.install_prereqs)

        # lk: set object names for CSS properties
        self.accept_button.setText(header)
        self.accept_button.setIcon(bicon)
        self.accept_button.setObjectName("InstallButton")

        self.action_button.setText(self.tr("Verify"))
        self.action_button.setIcon(qta_icon("fa.check"))

        self.setCentralWidget(install_widget)

    def showEvent(self, a0: QShowEvent) -> None:
        if a0.spontaneous():
            return super().showEvent(a0)
        self.install_dir_save_callback(self.install_dir_edit.text())
        super().showEvent(a0)

    def execute(self):
        if self.__options.silent:
            self.get_download_info()
        else:
            self.action_handler()
            self.open()

    @Slot(int)
    def reset_install_dir(self, index: int):
        if not self.rgame.is_installed:
            platform = self.ui.platform_combo.itemText(index)
            default_dir = self.core.get_default_install_dir(platform)
            self.install_dir_edit.setText(default_dir)

    @Slot(int)
    def check_incompatible_platform(self, index: int):
        platform = self.ui.platform_combo.itemText(index)
        if platform == "Mac" and pf.system() != "Darwin":
            self.error_box(
                self.tr("Warning"),
                self.tr("You will not be able to run the game if you select <b>{}</b> as platform").format(platform),
            )
        else:
            self.error_box()

    def get_options(self):
        base_path = os.path.join(self.install_dir_edit.text(), ".overlay" if self.__options.overlay else "")
        self.__options.base_path = "" if self.rgame.is_installed else base_path
        self.__options.platform = self.ui.platform_combo.currentText()
        self.__options.create_shortcut = self.ui.shortcut_check.isChecked()
        self.__options.max_workers = self.advanced.ui.max_workers_spin.value()
        self.__options.shared_memory = self.advanced.ui.max_memory_spin.value()
        self.__options.order_opt = self.advanced.ui.dl_optimizations_check.isChecked()
        self.__options.force = self.advanced.ui.force_download_check.isChecked()
        self.__options.ignore_space = self.advanced.ui.ignore_space_check.isChecked()
        self.__options.no_install = self.advanced.ui.download_only_check.isChecked()
        self.__options.install_prereqs = self.advanced.ui.install_prereqs_check.isChecked()
        self.__options.install_tag = self.selectable.install_tags()
        self.__options.reset_sdl = True

    def get_download_info(self):
        self.__download = None
        info_worker = InstallInfoWorker(self.core, self.__options)
        info_worker.signals.result.connect(self.on_worker_result)
        info_worker.signals.failed.connect(self.on_worker_failed)
        self.threadpool.start(info_worker)

    def action_handler(self):
        self.error_box()
        message = self.tr("Updating...")
        font = self.font()
        font.setItalic(True)
        self.ui.download_size_text.setText(message)
        self.ui.download_size_text.setFont(font)
        self.ui.install_size_text.setText(message)
        self.ui.install_size_text.setFont(font)
        self.setActive(True)
        self.options_changed = False
        self.get_options()
        self.get_download_info()

    @Slot()
    def option_changed(self):
        self.options_changed = True
        self.accept_button.setEnabled(False)
        self.action_button.setEnabled(not self.active())

    def install_dir_edit_callback(self, path: str) -> Tuple[bool, str, int]:
        self.option_changed()
        return True, path, IndicatorReasonsCommon.VALID

    def install_dir_save_callback(self, path: str):
        if not os.path.exists(path):
            return
        _, _, free_space = shutil.disk_usage(path)
        self.ui.avail_space.setText(format_size(free_space))

    def non_reload_option_changed(self, option: str):
        if option == "download_only":
            self.__options.no_install = self.advanced.ui.download_only_check.isChecked()
        elif option == "shortcut":
            QSettings().setValue("create_shortcut", self.ui.shortcut_check.isChecked())
            self.__options.create_shortcut = self.ui.shortcut_check.isChecked()
        elif option == "install_prereqs":
            self.__options.install_prereqs = self.advanced.ui.install_prereqs_check.isChecked()

    @staticmethod
    def same_platform(download: InstallDownloadModel) -> bool:
        platform = download.igame.platform
        if pf.system() == "Windows":
            return platform in {"Windows", "Win32"}
        elif pf.system() == "Darwin":
            return platform == "Mac"
        else:
            return False

    @Slot(InstallDownloadModel)
    def on_worker_result(self, download: InstallDownloadModel):
        self.setActive(False)
        self.__download = download
        download_size = download.analysis.dl_size
        install_size = download.analysis.install_size
        # install_size = self.dl_item.download.analysis.disk_space_delta
        bold_font = self.font()
        bold_font.setBold(True)
        italic_font = self.font()
        italic_font.setItalic(True)
        if download_size or (not download_size and (download.game.is_dlc or download.repair)):
            self.ui.download_size_text.setText(format_size(download_size))
            self.ui.download_size_text.setFont(bold_font)
            self.accept_button.setEnabled(not self.options_changed)
        else:
            self.ui.download_size_text.setText(self.tr("Game already installed"))
            self.ui.download_size_text.setFont(italic_font)
        self.ui.install_size_text.setText(format_size(install_size))
        self.ui.install_size_text.setFont(bold_font)
        self.action_button.setEnabled(self.options_changed)
        has_prereqs = bool(download.igame.prereq_info) and not download.igame.prereq_info.get("installed", False)
        if has_prereqs:
            prereq_name = download.igame.prereq_info.get("name", "")
            prereq_path = os.path.split(download.igame.prereq_info.get("path", ""))[-1]
            prereq_desc = prereq_name if prereq_name else prereq_path
            self.advanced.ui.install_prereqs_check.setText(self.tr("Also install: {}").format(prereq_desc))
        else:
            self.advanced.ui.install_prereqs_check.setText("")
        # Offer to install prerequisites only on same platforms
        self.advanced.ui.install_prereqs_label.setEnabled(has_prereqs)
        self.advanced.ui.install_prereqs_check.setEnabled(has_prereqs)
        self.advanced.ui.install_prereqs_check.setChecked(has_prereqs and self.same_platform(download))
        if self.__options.silent:
            self.accept()

    def on_worker_failed(self, message: str):
        self.setActive(False)
        error_text = self.tr("Error")
        self.ui.download_size_text.setText(error_text)
        self.ui.install_size_text.setText(error_text)
        self.error_box(error_text, message)
        self.action_button.setEnabled(self.options_changed)
        self.accept_button.setEnabled(False)
        if self.__options.silent:
            self.open()

    def error_box(self, label: str = "", message: str = ""):
        self.ui.warning_label.setVisible(bool(label))
        self.ui.warning_label.setText(label)
        self.ui.warning_text.setVisible(bool(message))
        self.ui.warning_text.setText(message)

    def done_handler(self):
        self.threadpool.clear()
        self.threadpool.waitForDone()
        self.result_ready.emit(self.__queue_item)

    # lk: __download is already set at this point so just do nothing.
    def accept_handler(self):
        self.__queue_item = InstallQueueItemModel(options=self.__options, download=self.__download)

    def reject_handler(self):
        self.__queue_item = InstallQueueItemModel(options=self.__options, download=None)


class TagCheckBox(QCheckBox):
    def __init__(self, text, desc, tags: List[str], parent=None):
        super(TagCheckBox, self).__init__(parent)
        self.setText(text)
        self.setToolTip(desc)
        self.tags = tags

    def isChecked(self) -> Union[bool, List[str]]:
        return self.tags if super(TagCheckBox, self).isChecked() else False



# -*- coding: utf-8 -*-

################################################################################
## Form generated from reading UI file 'install_dialog_advanced.ui'
##
## Created by: Qt User Interface Compiler version 6.7.2
##
## WARNING! All changes made in this file will be lost when recompiling UI file!
################################################################################

from PySide6.QtCore import (QCoreApplication, QDate, QDateTime, QLocale,
    QMetaObject, QObject, QPoint, QRect,
    QSize, QTime, QUrl, Qt)
from PySide6.QtGui import (QBrush, QColor, QConicalGradient, QCursor,
    QFont, QFontDatabase, QGradient, QIcon,
    QImage, QKeySequence, QLinearGradient, QPainter,
    QPalette, QPixmap, QRadialGradient, QTransform)
from PySide6.QtWidgets import (QApplication, QCheckBox, QFormLayout, QHBoxLayout,
    QLabel, QSizePolicy, QSpinBox, QWidget)

class Ui_InstallDialogAdvanced(object):
    def setupUi(self, InstallDialogAdvanced):
        if not InstallDialogAdvanced.objectName():
            InstallDialogAdvanced.setObjectName(u"InstallDialogAdvanced")
        InstallDialogAdvanced.resize(379, 208)
        InstallDialogAdvanced.setWindowTitle(u"InstallDialogAdvanced")
        self.install_dialog_advanced_layout = QFormLayout(InstallDialogAdvanced)
        self.install_dialog_advanced_layout.setObjectName(u"install_dialog_advanced_layout")
        self.max_workers_label = QLabel(InstallDialogAdvanced)
        self.max_workers_label.setObjectName(u"max_workers_label")
        self.max_workers_label.setAlignment(Qt.AlignmentFlag.AlignLeading|Qt.AlignmentFlag.AlignLeft|Qt.AlignmentFlag.AlignVCenter)

        self.install_dialog_advanced_layout.setWidget(0, QFormLayout.LabelRole, self.max_workers_label)

        self.max_workers_layout = QHBoxLayout()
        self.max_workers_layout.setObjectName(u"max_workers_layout")
        self.max_workers_spin = QSpinBox(InstallDialogAdvanced)
        self.max_workers_spin.setObjectName(u"max_workers_spin")
        sizePolicy = QSizePolicy(QSizePolicy.Policy.Maximum, QSizePolicy.Policy.Fixed)
        sizePolicy.setHorizontalStretch(0)
        sizePolicy.setVerticalStretch(0)
        sizePolicy.setHeightForWidth(self.max_workers_spin.sizePolicy().hasHeightForWidth())
        self.max_workers_spin.setSizePolicy(sizePolicy)

        self.max_workers_layout.addWidget(self.max_workers_spin)

        self.max_workers_info = QLabel(InstallDialogAdvanced)
        self.max_workers_info.setObjectName(u"max_workers_info")
        font = QFont()
        font.setItalic(True)
        self.max_workers_info.setFont(font)

        self.max_workers_layout.addWidget(self.max_workers_info)


        self.install_dialog_advanced_layout.setLayout(0, QFormLayout.ItemRole.FieldRole, self.max_workers_layout)

        self.max_memory_label = QLabel(InstallDialogAdvanced)
        self.max_memory_label.setObjectName(u"max_memory_label")
        self.max_memory_label.setAlignment(Qt.AlignmentFlag.AlignLeading|Qt.AlignmentFlag.AlignLeft|Qt.AlignmentFlag.AlignVCenter)

        self.install_dialog_advanced_layout.setWidget(1, QFormLayout.LabelRole, self.max_memory_label)

        self.max_memory_layout = QHBoxLayout()
        self.max_memory_layout.setObjectName(u"max_memory_layout")
        self.max_memory_spin = QSpinBox(InstallDialogAdvanced)
        self.max_memory_spin.setObjectName(u"max_memory_spin")
        sizePolicy.setHeightForWidth(self.max_memory_spin.sizePolicy().hasHeightForWidth())
        self.max_memory_spin.setSizePolicy(sizePolicy)
        self.max_memory_spin.setMinimum(0)
        self.max_memory_spin.setMaximum(10240)
        self.max_memory_spin.setSingleStep(128)
        self.max_memory_spin.setValue(1024)

        self.max_memory_layout.addWidget(self.max_memory_spin)

        self.max_memory_info = QLabel(InstallDialogAdvanced)
        self.max_memory_info.setObjectName(u"max_memory_info")
        self.max_memory_info.setFont(font)

        self.max_memory_layout.addWidget(self.max_memory_info)


        self.install_dialog_advanced_layout.setLayout(1, QFormLayout.ItemRole.FieldRole, self.max_memory_layout)

        self.install_prereqs_label = QLabel(InstallDialogAdvanced)
        self.install_prereqs_label.setObjectName(u"install_prereqs_label")

        self.install_dialog_advanced_layout.setWidget(2, QFormLayout.LabelRole, self.install_prereqs_label)

        self.install_prereqs_check = QCheckBox(InstallDialogAdvanced)
        self.install_prereqs_check.setObjectName(u"install_prereqs_check")
        self.install_prereqs_check.setFont(font)
        self.install_prereqs_check.setText(u"")
        self.install_prereqs_check.setChecked(False)

        self.install_dialog_advanced_layout.setWidget(2, QFormLayout.ItemRole.FieldRole, self.install_prereqs_check)

        self.dl_optimizations_label = QLabel(InstallDialogAdvanced)
        self.dl_optimizations_label.setObjectName(u"dl_optimizations_label")

        self.install_dialog_advanced_layout.setWidget(3, QFormLayout.LabelRole, self.dl_optimizations_label)

        self.dl_optimizations_check = QCheckBox(InstallDialogAdvanced)
        self.dl_optimizations_check.setObjectName(u"dl_optimizations_check")
        self.dl_optimizations_check.setText(u"")
        self.dl_optimizations_check.setChecked(False)

        self.install_dialog_advanced_layout.setWidget(3, QFormLayout.ItemRole.FieldRole, self.dl_optimizations_check)

        self.force_download_label = QLabel(InstallDialogAdvanced)
        self.force_download_label.setObjectName(u"force_download_label")

        self.install_dialog_advanced_layout.setWidget(4, QFormLayout.LabelRole, self.force_download_label)

        self.force_download_check = QCheckBox(InstallDialogAdvanced)
        self.force_download_check.setObjectName(u"force_download_check")
        self.force_download_check.setText(u"")

        self.install_dialog_advanced_layout.setWidget(4, QFormLayout.ItemRole.FieldRole, self.force_download_check)

        self.ignore_space_label = QLabel(InstallDialogAdvanced)
        self.ignore_space_label.setObjectName(u"ignore_space_label")

        self.install_dialog_advanced_layout.setWidget(5, QFormLayout.LabelRole, self.ignore_space_label)

        self.ignore_space_check = QCheckBox(InstallDialogAdvanced)
        self.ignore_space_check.setObjectName(u"ignore_space_check")
        self.ignore_space_check.setFont(font)

        self.install_dialog_advanced_layout.setWidget(5, QFormLayout.ItemRole.FieldRole, self.ignore_space_check)

        self.download_only_label = QLabel(InstallDialogAdvanced)
        self.download_only_label.setObjectName(u"download_only_label")

        self.install_dialog_advanced_layout.setWidget(6, QFormLayout.LabelRole, self.download_only_label)

        self.download_only_check = QCheckBox(InstallDialogAdvanced)
        self.download_only_check.setObjectName(u"download_only_check")
        self.download_only_check.setFont(font)

        self.install_dialog_advanced_layout.setWidget(6, QFormLayout.ItemRole.FieldRole, self.download_only_check)


        self.retranslateUi(InstallDialogAdvanced)
    # setupUi

    def retranslateUi(self, InstallDialogAdvanced):
        self.max_workers_label.setText(QCoreApplication.translate("InstallDialogAdvanced", u"Max workers", None))
        self.max_workers_info.setText(QCoreApplication.translate("InstallDialogAdvanced", u"Less is slower. (0: Default)", None))
        self.max_memory_label.setText(QCoreApplication.translate("InstallDialogAdvanced", u"Max shared memory", None))
        self.max_memory_spin.setSuffix(QCoreApplication.translate("InstallDialogAdvanced", u"MiB", None))
        self.max_memory_info.setText(QCoreApplication.translate("InstallDialogAdvanced", u"Less is slower (0: Default)", None))
        self.install_prereqs_label.setText(QCoreApplication.translate("InstallDialogAdvanced", u"Install prerequisites", None))
        self.dl_optimizations_label.setText(QCoreApplication.translate("InstallDialogAdvanced", u"Enable reordering", None))
        self.force_download_label.setText(QCoreApplication.translate("InstallDialogAdvanced", u"Force redownload", None))
        self.ignore_space_label.setText(QCoreApplication.translate("InstallDialogAdvanced", u"Ignore free space", None))
        self.ignore_space_check.setText(QCoreApplication.translate("InstallDialogAdvanced", u"Use with caution!", None))
        self.download_only_label.setText(QCoreApplication.translate("InstallDialogAdvanced", u"Download only", None))
        self.download_only_check.setText(QCoreApplication.translate("InstallDialogAdvanced", u"Do not try to install.", None))
        pass
    # retranslateUi

